@@ -3,7 +3,7 @@
 
 # Morituri - for those about to RIP
 
-# Copyright (C) 2009 Thomas Vander Stichele
+# Copyright (C) 2009, 2010, 2011 Thomas Vander Stichele
 
 # This file is part of morituri.
 # 
@@ -105,6 +105,7 @@ def musicbrainz(discid):
     #import musicbrainz2.disc as mbdisc
     import musicbrainz2.webservice as mbws
 
+    results = []
 
     # Setup a Query object.
     service = mbws.WebService()
@@ -172,11 +173,27 @@ class Program(log.Loggable):
             'result')
         return path
 
+    def loadDevice(self, device):
+        """
+        Load the given device.
+        """
+        os.system('eject -t %s' % device)
+
+    def ejectDevice(self, device):
+        """
+        Eject the given device.
+        """
+        os.system('eject %s' % device)
+
     def unmountDevice(self, device):
         """
         Unmount the given device if it is mounted, as happens with automounted
         data tracks.
+
+        If the given device is a symlink, the target will be checked.
         """
+        device = os.path.realpath(device)
+        self.debug('possibly unmount real path %r' % device)
         proc = open('/proc/mounts').read()
         if device in proc:
             print 'Device %s is mounted, unmounting' % device
@@ -248,6 +265,8 @@ class Program(log.Loggable):
         assert type(outdir) is unicode, "%r is not unicode" % outdir
         assert type(template) is unicode, "%r is not unicode" % template
 
+        # the template is similar to grip, except for %s/%S
+
         # returns without extension
 
         v = {}
@@ -286,6 +305,22 @@ class Program(log.Loggable):
 
         return os.path.join(outdir, template % v)
 
+    def getCDDB(self, cddbdiscid):
+        """
+        @param cddbdiscid: list of id, tracks, offsets, seconds
+
+        @rtype: str
+        """
+        # FIXME: convert to nonblocking?
+        import CDDB
+        code, md = CDDB.query(cddbdiscid) 
+        self.debug('CDDB query result: %r, %r', code, md)
+        if code == 200:
+            return md['title']
+
+        return None
+
+
     def getMusicBrainz(self, ittoc, mbdiscid):
         # look up disc on musicbrainz
         ret = None
@@ -323,19 +358,21 @@ class Program(log.Loggable):
 
         @rtype: L{gst.TagList}
         """
-        artist = u'Unknown Artist'
+        trackArtist = u'Unknown Artist'
+        albumArtist = u'Unknown Artist'
         disc = u'Unknown Disc'
         title = u'Unknown Track'
 
         if self.metadata:
-            artist = self.metadata.artist
+            trackArtist = self.metadata.artist
+            albumArtist = self.metadata.artist
             disc = self.metadata.title
             mbidAlbum = self.metadata.mbid
             mbidTrackAlbum = self.metadata.mbidArtist
 
             if number > 0:
                 try:
-                    artist = self.metadata.tracks[number - 1].artist
+                    trackArtist = self.metadata.tracks[number - 1].artist
                     title = self.metadata.tracks[number - 1].title
                     mbidTrack = self.metadata.tracks[number - 1].mbid
                     mbidTrackArtist = self.metadata.tracks[number - 1].mbidArtist
@@ -353,7 +390,9 @@ class Program(log.Loggable):
 
         # gst-python 0.10.15.1 does not handle unicode -> utf8 string conversion
         # see http://bugzilla.gnome.org/show_bug.cgi?id=584445
-        ret[gst.TAG_ARTIST] = artist.encode('utf-8')
+        if self.metadata.various:
+            ret["album-artist"] = albumArtist.encode('utf-8')
+        ret[gst.TAG_ARTIST] = trackArtist.encode('utf-8')
         ret[gst.TAG_TITLE] = title.encode('utf-8')
         ret[gst.TAG_ALBUM] = disc.encode('utf-8')
 
@@ -422,7 +461,8 @@ class Program(log.Loggable):
 
         return trackResult.testcrc == t.checksum
 
-    def ripTrack(self, runner, trackResult, offset, device, profile, taglist):
+    def ripTrack(self, runner, trackResult, offset, device, profile, taglist,
+        what=None):
         """
         @param trackResult: the object to store information in.
         @type  trackResult: L{result.TrackResult}
@@ -439,13 +479,16 @@ class Program(log.Loggable):
         if not os.path.exists(dirname):
             os.makedirs(dirname)
 
+        if not what:
+            what='track %d' % (trackResult.number, )
+
         t = cdparanoia.ReadVerifyTrackTask(trackResult.filename,
             self.result.table, start, stop,
             offset=offset,
             device=device,
             profile=profile,
-            taglist=taglist)
-        t.description = 'Reading Track %d' % trackResult.number 
+            taglist=taglist,
+            what=what)
 
         runner.run(t)
 
@@ -489,31 +532,26 @@ class Program(log.Loggable):
             self.warning('No AccurateRip responses, cannot verify.')
             return
 
-        response = None # track which response matches, for all tracks
-
         # now loop to match responses
         for i, csum in enumerate(checksums):
             trackResult = self.result.getTrackResult(i + 1)
 
             confidence = None
+            response = None
 
             # match against each response's checksum for this track
             for j, r in enumerate(responses):
                 if "%08x" % csum == r.checksums[i]:
-                    if not response:
-                        response = r
-                    else:
-                        assert r == response, \
-                            "checksum %s for %d matches wrong response %d, "\
-                            "checksum %s" % (
-                                csum, i + 1, j + 1, response.checksums[i])
-                    self.debug("Track: %02d matched in AccurateRip database",
-                        i + 1)
+                    response = r
+                    self.debug(
+                        "Track %02d matched response %d of %d in "
+                        "AccurateRip database",
+                        i + 1, j + 1, len(responses))
                     trackResult.accurip = True
                     # FIXME: maybe checksums should be ints
-                    trackResult.ARDBCRC = int(response.checksums[i], 16)
+                    trackResult.ARDBCRC = int(r.checksums[i], 16)
                     # arsum = csum
-                    confidence = response.confidences[i]
+                    confidence = r.confidences[i]
                     trackResult.ARDBConfidence = confidence
 
             if not trackResult.accurip:
@@ -577,10 +615,9 @@ class Program(log.Loggable):
         return res
 
     def writeCue(self, discName):
-        self.debug('write .cue file')
         assert self.result.table.canCue()
-
         cuePath = '%s.cue' % discName
+        self.debug('write .cue file to %s', cuePath)
         handle = open(cuePath, 'w')
         # FIXME: do we always want utf-8 ?
         handle.write(self.result.table.cue().encode('utf-8'))
