@@ -11,7 +11,8 @@ These are, in order of decreasing verbosity: log, debug, info, warning, error.
 
 This module provides a Loggable class for objects, as well as various
 convenience methods for logging in general, and for logging with Twisted
-and failures in particular.
+and failures in particular. It is written such a way that it can be reused in
+different projects to provide a controllable logging mechanism.
 
 Maintainer: U{Thomas Vander Stichele <thomas at apestaart dot org>}
 """
@@ -22,6 +23,7 @@ import os
 import fnmatch
 import time
 import types
+import logging
 import traceback
 
 # environment variables controlling levels for each category
@@ -206,7 +208,7 @@ def scrubFilename(filename):
     return filename
 
 
-def getFileLine(where=-1):
+def getFileLine(where=-1, targetModule=None):
     """
     Return the filename and line number for the given location.
 
@@ -217,6 +219,9 @@ def getFileLine(where=-1):
     @param where: how many frames to go back up, or function
     @type  where: int (negative) or function
 
+    @param targetModule: continue going up the stack until that module is seen
+    @type  targetModule: str or None
+
     @return: tuple of (file, line)
     @rtype:  tuple of (str, int)
     """
@@ -246,6 +251,14 @@ def getFileLine(where=-1):
     if not co:
         return "<unknown file>", 0
 
+    if targetModule:
+        while stackFrame:
+            co = stackFrame.f_code
+            lineno = stackFrame.f_lineno
+            if targetModule in co.co_filename:
+                break
+            stackFrame = stackFrame.f_back
+
     return scrubFilename(co.co_filename), lineno
 
 
@@ -835,7 +848,7 @@ class Loggable:
             if hasattr(self, name):
                 return getattr(self, name)
 
-        return None
+        return '<' + self.__class__.__name__ + '>'
 
 # Twisted helper stuff
 
@@ -977,3 +990,67 @@ class TwistedLogObserver(Loggable):
 
     def clearIgnores(self):
         self._ignoreErrors = []
+
+
+def logLevelToStdLevel(level):
+    t = {ERROR : logging.CRITICAL,
+         WARN : logging.ERROR,
+         INFO : logging.WARNING,
+         DEBUG : logging.INFO,
+         LOG : logging.DEBUG}
+    return t.get(level, logging.NOTSET)
+
+
+def stdLevelToLogLevel(level):
+    t = {logging.CRITICAL: ERROR,
+         logging.ERROR: WARN,
+         logging.WARNING: INFO,
+         logging.INFO: DEBUG,
+         logging.DEBUG: LOG}
+    return t.get(level, None)
+
+
+def adaptStandardLogging(loggerName, logCategory, targetModule):
+    """
+    Make a logger from the standard library log through the Flumotion logging
+    system.
+
+    @param loggerName: The standard logger to adapt, e.g. 'library.module'
+    @type loggerName: str
+    @param logCategory: The Flumotion log category to use when reporting output
+                        from the standard logger, e.g. 'librarymodule'
+    @type logCategory: str
+    @param targetModule: The name of the module that the logging should look
+                         like it's coming from. Use this if you don't want to
+                         see the file names and line numbers of the library
+                         who's logger you are adapting.
+    @type targetModule: str or None
+    """
+    logger = logging.getLogger(loggerName)
+    # if there is already a FluHandler, exit
+    if map(lambda h: isinstance(h, LogHandler), logger.handlers):
+        return
+    logger.setLevel(logLevelToStdLevel(getCategoryLevel(logCategory)))
+    logger.addHandler(LogHandler(logCategory, targetModule))
+
+
+class LogHandler(logging.Handler):
+    """
+    A standard library logging handler that logs through the log system of this
+    module.
+    """
+
+    def __init__(self, logCategory, targetModule):
+        logging.Handler.__init__(self)
+        self.logCategory = logCategory
+        self.targetModule = targetModule
+
+    def emit(self, record):
+        level = stdLevelToLogLevel(record.levelno)
+        if _canShortcutLogging(self.logCategory, level):
+            return
+
+        filename, lineno = getFileLine(-1, self.targetModule)
+        doLog(level, None, self.logCategory,
+              self.format(record), None, 0,
+              scrubFilename(filename), lineno)
